--------------------------------------------------------------------------------
-- Beach Ball by Holzchopf
-- v1.1 - 28 Sep 2011	- Updated hierarchy
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.beachball={}
holzchopf.beachball.ball={}

-- load ressources
holzchopf.beachball.gfx_wpn=loadgfx("holzchopf/beachball.png")
setmidhandle(holzchopf.beachball.gfx_wpn)
holzchopf.beachball.sfx_hit=loadsfx("boing.ogg")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.beachball.id=addweapon("holzchopf.beachball","Beach Ball",holzchopf.beachball.gfx_wpn,0,3)

function holzchopf.beachball.draw()
	if weapon_shots==0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setrotation(getplayerrotation(0))
		drawinhand(holzchopf.beachball.gfx_wpn,3,-3)
		hudcrosshair(3,-3)
		-- HUD chargebar
		if weapon_charge>0 then
			hudchargebar(weapon_charge,100)
		end
	end
end

function holzchopf.beachball.attack(attack)
	-- charge
	if weapon_shots==0 and attack==1 then
		weapon_charge=weapon_charge+1
	end
	-- throw
	if (weapon_charge>0 and weapon_shots==0 and attack==0) or (weapon_charge>=100 and weapon_shots==0) then
		-- throw stone
		id=createprojectile(holzchopf.beachball.ball.id)
		projectiles[id]={}
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+getplayerdirection(0)*3
		projectiles[id].y=getplayery(0)-3
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*weapon_charge/10
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*weapon_charge/10
		projectiles[id].angle=getplayerrotation(0)
		projectiles[id].rotation=getplayerdirection(0)
		-- ignore player collision (only terrain collision)
		projectiles[id].pc=0
		-- nitial coll check
		if collision(holzchopf.beachball.gfx_wpn,projectiles[id].x,projectiles[id].y,1,0)==1 then
			freeprojectile(id)
			weapon_charge=0
			return
		end
		-- use
		useweapon(0)
		playercontrol(0)
		weapon_shots=1
		weapon_timer=1
		changeturntime(15-math.floor(getframesleft()/50))
	end
end

--------------------------------------------------------------------------------
-- Projectile
--------------------------------------------------------------------------------
holzchopf.beachball.ball.id=addprojectile("holzchopf.beachball.ball")

function holzchopf.beachball.ball.draw(id)
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(projectiles[id].angle)
	drawimage(holzchopf.beachball.gfx_wpn,projectiles[id].x,projectiles[id].y)
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	scroll(projectiles[id].x,projectiles[id].y)
	hudinfo("Hit [fire] to detonate the beach ball!")
end

function holzchopf.beachball.ball.update(id)
	-- detonate
	if keydown(key_attack)==0 then weapon_timer=0 end
	if (keydown(key_attack)==1 and weapon_timer==0) or getframesleft()<=0 then
		-- Splatter
		playsound(sfx_splatter2)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,200,100)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,90,1)
		-- Crater
		grey=math.random(0,40)
		terrainalphaimage(gfx_crater200,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/2)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- activate player collision
		if collision(holzchopf.beachball.gfx_wpn,projectiles[id].x+msubx,projectiles[id].y+msuby,0,1)==0 then
			projectiles[id].pc=1
		end
		-- collision check
		if collision(holzchopf.beachball.gfx_wpn,projectiles[id].x+msubx,projectiles[id].y+msuby,1,projectiles[id].pc)==1 then
			if playercollision()~=0 then
				local v=math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)
				if v>2 then
					playerdamage(playercollision(0),v-2)
					playsound(sfx_splatter2)
				end
			end
			-- define angle
			local mx=0
			local my=0
			for a=0,350,10 do
				local ix=math.cos(math.rad(a))*10
				local iy=math.sin(math.rad(a))*10
				if collision(col2x2,projectiles[id].x+ix,projectiles[id].y+iy,1,projectiles[id].pc)==1 then
					mx=mx+ix
					my=my+iy
				end
			end
			local an=math.atan2(mx,my)
			local ai=math.atan2(projectiles[id].sx,projectiles[id].sy)
			local ao=an*2-ai+math.pi
--drawline(projectiles[id].x,projectiles[id].y,projectiles[id].x+math.sin(ao)*100,projectiles[id].y+math.cos(ao)*100)
			local s=math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)
			projectiles[id].sx=math.sin(ao)*s*0.8
			projectiles[id].sy=math.cos(ao)*s*0.8
			projectiles[id].rotation=projectiles[id].rotation*0.1 +math.sin(ai-an)*0.9
			-- Effects
			if math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)>2 then
				playsound(holzchopf.beachball.sfx_hit)
			end
			break
		elseif projectiles[id].y+projectiles[id].sy>getwatery()-48 then
			projectiles[id].sy=-projectiles[id].sy*0.4
			projectiles[id].sx=projectiles[id].sx*0.7
			local sgn=0
			if projectiles[id].sx>0 then sgn=1 else sgn=-1 end
			projectiles[id].rotation=projectiles[id].rotation*0.1 +sgn*0.9
			-- Effects
			if math.sqrt(projectiles[id].sx^2+projectiles[id].sy^2)>2 then
				particle(p_waterhit,projectiles[id].x,projectiles[id].y)
				if math.random(1,2)==1 then
					playsound(sfx_hitwater2)
				else
					playsound(sfx_hitwater3)
				end
			end
			break
		else
			-- move
			projectiles[id].x=projectiles[id].x+msubx
			projectiles[id].y=projectiles[id].y+msuby
			projectiles[id].angle=projectiles[id].angle+projectiles[id].rotation*3*math.sqrt(msubx^2+msuby^2)
			if i==1 then
				-- wind and gravity
				projectiles[id].sx=projectiles[id].sx+getwind()*2
				projectiles[id].sy=projectiles[id].sy+getgravity()
				projectiles[id].rotation=projectiles[id].rotation*0.96
			end
		end
	end
end